var map;
var mapMarker;
var formPrefix;

function loadGoogleMaps() {

	var mapElement = document.getElementById('admin-map');
	var src = `https://maps.googleapis.com/maps/api/js?key=${mapElement.dataset.key}&callback=initMap`;
	if (document.querySelectorAll(`script[src="${src}"]`).length) {
		initMap();
	} else {
		var googleMapsLibrary = document.createElement('script');
		googleMapsLibrary.src = src;
		googleMapsLibrary.defer = true;
		document.head.appendChild(googleMapsLibrary);
	}


}


window.initMap = function () {


	formPrefix = document.getElementById('Form_ItemEditForm_Latitude') ? 'Form_ItemEditForm_' : 'Form_EditForm_';

    var latField = document.getElementById(formPrefix + 'Latitude');
    var lngField = document.getElementById(formPrefix + 'Longitude');
    var zoomLevel = 14;

	const mapElement = document.getElementById('admin-map');

    if (!latField.value || !lngField.value) {
        latField.value = parseFloat(mapElement.dataset.defaultLatitude);
        lngField.value = parseFloat(mapElement.dataset.defaultLongitude);
    }

	console.log(latField.value);

    var center = new google.maps.LatLng(latField.value, lngField.value);
    map = new google.maps.Map(mapElement, {
        center: center,
        zoom: zoomLevel,
        scrollwheel: false
    });

    mapMarker = new google.maps.Marker({
        map: map,
        position: center,
        draggable: true
    });
    mapMarker.addListener('dragend', function() {
        setLatLng(
            mapMarker.getPosition().lat(),
            mapMarker.getPosition().lng()
        );
    });
}


function setCoords() {
	fetch('https://geocode.maps.co/search?q=' + encodeURIComponent(getAddress()))
		.then(response => {
			if (!response.ok) {
				return Promise.reject(response);
			}
			return response.json();
		})
		.then(data => {
			if (!Array.isArray(data) || !data.length) {
				return Promise.reject('No data');
			}
			const latitude = parseFloat(data[0].lat);
			const longitude = parseFloat(data[0].lon);
			setMarker(latitude, longitude);
			setLatLng(latitude, longitude);
		})
		.catch(error => {
			console.error('Could not fetch address coordinates');
		});
}


function setMarker(lat, lng) {
    var center = new google.maps.LatLng(lat, lng);
    mapMarker.setPosition(center);
    map.setCenter(center);
    map.setZoom(14);
}


function setLatLng(lat, lng) {
    document.getElementById(formPrefix + 'Latitude').value = lat;
    document.getElementById(formPrefix + 'Longitude').value = lng;
}


function getAddress() {
    var fields = [
        document.getElementById(formPrefix + 'Address1')?.value,
        document.getElementById(formPrefix + 'Address2')?.value,
        document.getElementById(formPrefix + 'City')?.value,
        document.getElementById(formPrefix + 'Postcode')?.value,
        document.getElementById(formPrefix + 'State')?.value,
        document.getElementById(formPrefix + 'Country')?.value
    ];
    var address = '';
    for (var i = 0; i < fields.length; i++) {
        if (fields[i]) {
            address += fields[i] + ' ';
        }
    }
    return address;
}


(function ($) {
    $(document).ready(function() {

        jQuery('.cms-container').on('click', '.location-tab-with-map .ui-tabs-anchor', function () {
			loadGoogleMaps();
        });

        jQuery('.cms-container').on('click', '#set-map-coords', function () {
            setCoords();
        });

        $.entwine(function($) {
            $("div.admin-google-map").entwine({
                onmatch: function() {
					loadGoogleMaps();
                    this._super();
                }
            });
        });


    });
})(jQuery);
