<?php

namespace Fastrack\Core\Admin;

use Fastrack\Core\Model\CalendarEvent;
use Fastrack\Core\Model\CalendarEventCategory;
use SilverStripe\Admin\ModelAdmin;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\Form;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;

class CalendarEventAdmin extends ModelAdmin
{
	private static string $menu_title = 'Events';

	private static string $url_segment = 'events';

	private static string $menu_icon_class = 'font-icon-p-event';

	private static array $managed_models = [
		CalendarEvent::class => ['title' => 'Events'],
		CalendarEventCategory::class => ['title' => 'Categories'],
	];

	public function getEditForm($id = null, $fields = null): Form
	{
		$form = parent::getEditForm($id, $fields);
		$this->updateAddButton($form->Fields(), CalendarEvent::class, 'Add event');
		$this->updateAddButton($form->Fields(), CalendarEventCategory::class, 'Add category');

		$form->Fields()->dataFieldByName($this->sanitiseClassName(CalendarEvent::class))
			?->getConfig()
			->addComponent(GridFieldOrderableRows::create());
		return $form;
	}

	public function subsiteCMSShowInMenu(): bool
	{
		return true;
	}

	private function updateAddButton(FieldList $fields, string $class, string $label): void
	{
		/** @var ?GridField $gridField */
		/** @var ?GridFieldAddNewButton $button */
		$gridField = $fields->dataFieldByName($this->sanitiseClassName($class));
		$button = $gridField?->getConfig()->getComponentByType(GridFieldAddNewButton::class);
		$button?->setButtonName($label);
	}

}
