<?php

namespace Fastrack\Core\Admin;

use DirectoryIterator;
use League\CommonMark\Exception\CommonMarkException;
use League\CommonMark\GithubFlavoredMarkdownConverter;
use SilverStripe\Admin\LeftAndMain;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Core\Path;
use SilverStripe\ORM\ArrayList;
use SilverStripe\View\ArrayData;

class DocumentationAdmin extends LeftAndMain
{
	private static string $menu_title = 'Documentation';
	private static string $url_segment = 'docs';
	private static string $menu_icon_class = 'font-icon-book';
	private static int $menu_priority = -1;

	private static array $allowed_actions = [
		'handleDocument',
	];

	private static array $url_handlers = [
		'//$Document!' => 'handleDocument',
	];

	const DOCS_DIR = 'app/docs/';

	public function index(HTTPRequest $request): HTTPResponse
	{
		if (!$request->param('Document')) {
			return $this->handleDocument($request);
		}
		return parent::index($request);
	}

	public function handleDocument(HTTPRequest $request): HTTPResponse
	{
		if (!Director::fileExists($this->getRequestedDocument())) {
			$toc = $this->getToC();
			if (!$toc->count()) {
				return $this->getResponse()->setStatusCode('500', 'No documentation found');
			}
			return $this->getResponse()->redirect(Controller::join_links(['/', $toc->first()->getField('Link')]));
		}
		return parent::index($request);
	}

	public function getToC(): ArrayList
	{
		$docsDir = Path::join([Director::baseFolder(), self::DOCS_DIR]);
		$toc = ArrayList::create();
		/** @var DirectoryIterator $file */
		foreach (new DirectoryIterator($docsDir) as $file) {
			if ($file->isDot()) continue;
			if ($file->isDir()) continue;
			if (pathinfo($file->getFilename(),  PATHINFO_EXTENSION) !== 'md') continue;
			$fileName = pathinfo($file->getFilename(),  PATHINFO_FILENAME);
			$nice = ucfirst(str_replace(['-', '.md'], [' ', ''], $fileName));
			$toc->push(ArrayData::create([
				'Link' => $this->Link($fileName),
				'Title' => $nice,
			]));
		}
		return $toc->sort('Title');
	}

	public function getDocument(): string
	{
		$content = "# Document not found \n\n Could not find the document you were looking for.";
		$fileName = $this->getRequestedDocument();
		if (Director::fileExists($fileName)) {
			$file = file_get_contents($fileName);
			if ($file) {
				$content = $file;
			}
		}
		$converter = new GithubFlavoredMarkdownConverter();
		try {
			return $converter->convert($content)->getContent();
		} catch (CommonMarkException) {
			return '<h1>Error</h1><p>Could not render document</p>';
		}
	}

	private function getRequestedDocument(): string
	{
		$document = $this->request->param('Document') ?? '';
		return Path::join([
			Director::baseFolder(),
			self::DOCS_DIR,
			$document . '.md',
		]);
	}

}
