<?php

namespace Fastrack\Core\Admin;

use Fastrack\Core\Api\Api;
use Fastrack\Core\Model\SharedFeature;
use Fastrack\Core\Model\Offer;
use Fastrack\Core\Model\Property;
use SilverStripe\Admin\ModelAdmin;
use SilverStripe\Core\Config\Config;
use SilverStripe\Forms\Form;
use SilverStripe\Forms\GridField\GridField;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;

class PropertyAdmin extends ModelAdmin
{
	private static string $menu_title = 'Properties';

	private static string $url_segment = 'properties';

	private static string $menu_icon = 'vendor/fastrack/core/client/images/property.png';

	/**
	 * Add extra models with config.yml (e.g. BookDirectFeature, RoomType)
	 */
	private static array $managed_models = [
		'property' => ['title' => 'Properties', 'dataClass' => Property::class],
		'offer' => ['title' => 'Offers', 'dataClass' => Offer::class],
		'feature' => ['title' => 'Features', 'dataClass' => SharedFeature::class],
	];


	public function getEditForm($id = null, $fields = null): Form
	{
		$form = parent::getEditForm($id, $fields);
		if ($this->modelClass === Property::class) {
			$this->enableSortableRows($form->Fields()->dataFieldByName('property'));
		}
		return $form;
	}

	public function subsiteCMSShowInMenu(): bool
	{
		return true;
	}

	private function enableSortableRows($gridField): void
	{
		if ($gridField instanceof GridField) {
			$gridField->getConfig()->addComponent(GridFieldOrderableRows::create());
		}
	}

}
