<?php

namespace Fastrack\Core\Api;

use SilverStripe\Blog\Model\Blog;
use SilverStripe\Blog\Model\BlogCategory;
use SilverStripe\Blog\Model\BlogPost;
use SilverStripe\Blog\Model\BlogTag;
use SilverStripe\CMS\Controllers\ContentController;
use SilverStripe\Control\Controller;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;

class BlogApi extends Controller
{

	/**
	 * @config
	 */
	private static int $excerpt_word_count = 30;

	private static array $allowed_actions = [
		'blog',
	];

	private static array $url_handlers = [
		'$BlogID!' => 'blog'
	];

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function blog(): HTTPResponse
	{
		/** @var Blog $blog */
		$blog = Blog::get()->byID(intval($this->getRequest()->param('BlogID')));
		if (!$blog) $this->httpError(404);

		$categories = $blog->Categories()->filterByCallback(function ($c) {return $c->BlogPosts()->count() > 0;});

		$data = [
			'id' => $blog->ID,
			'link' => $blog->Link(),
			'title' => $blog->Title,
			'content' => $blog->Content,
			'categories' => array_map('self::buildCategory', $categories->toArray()),
			'blogPosts' => array_map('self::buildBlogPost', $blog->getBlogPosts()->toArray()),
		];
		$this->extend('updateBuildBlog', $data, $blog);
		return Api::constructResponse($this, $data);

	}

	private function buildCategory(BlogCategory $category): array
	{
		$data = [
			'id' => $category->ID,
			'title' => $category->getTitle(),
		];
		$this->extend('updateBuildCategory', $data, $category);
		return $data;
	}

	private function buildTag(BlogTag $tag): array
	{
		$data = [
			'id' => $tag->ID,
			'title' => $tag->Title,
		];
		$this->extend('updateBuildTag', $data, $tag);
		return $data;
	}

	private function buildBlogPost(BlogPost $blogPost): array
	{
		$data = [
			'id' => $blogPost->ID,
			'title' => $blogPost->getTitle(),
			'excerpt' => $blogPost->Excerpt($this->config()->get('excerpt_word_count')),
			'content' => $blogPost->Content,
			'link' => $blogPost->Link(),
			'image' => Api::buildImage($blogPost->FeaturedImage()),
			'categories' => array_map('self::buildCategory', $blogPost->Categories()->toArray()),
			'tags' => array_map('self::buildTag', $blogPost->Tags()->toArray()),
			'publishDate' => $blogPost->PublishDate
		];
		$this->extend('updateBuildBlogPost', $data, $blogPost);
		return $data;
	}

	private function filterCategories(BlogCategory $category): bool
	{
		return $category->BlogPosts()->count() > 0;
	}

}
