<?php

namespace Fastrack\Core\Controller;

use Exception;
use Fastrack\Core\Api\Api;
use Fastrack\Core\Model\InstagramToken;
use Fastrack\Core\Util\HttpUtil;
use SilverStripe\Control\Controller;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;

class InstagramController extends Controller
{

	private static array $allowed_actions = [
		'auth',
		'accessToken',
	];

	private static array $url_handlers = [
		'$Username!/auth' => 'auth',
		'$Username!/access-token' => 'accessToken',
	];

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function auth(): HTTPResponse
	{
		try {
			$settings = $this->getSettings();
			$code = $this->getRequest()->getVar('code');
			$state = $this->getRequest()->getVar('state');
			$provider = $settings->getProvider();
			if (!$code) {
				$authUrl = $provider->getAuthorizationUrl(['scope' => ['user_profile', 'user_media']]);
				$settings->State = $provider->getState();
				$settings->write();
				return $this->getResponse()->redirect($authUrl);
			} else if (!$state || $state !== $settings->State) {
				throw new Exception('Invalid state');
			} else {
				$token = $provider->getAccessToken('authorization_code', ['code' => $code]);
				$longLivedToken = $provider->getLongLivedAccessToken($token);
				$settings->AccessToken = $token->getToken();
				$settings->RefreshToken = $longLivedToken->getToken();
				$settings->Expires = $longLivedToken->getExpires();
				$settings->ResourceOwnerId = $token->getResourceOwnerId();
				$settings->write();
				return Api::constructResponse($this, [
					'accessToken' => $settings->AccessToken,
					'refreshToken' => $settings->RefreshToken,
					'expires' => $settings->Expires,
					'resourceOwnerId' => $settings->ResourceOwnerId,
				]);
			}
		} catch (Exception $e) {
			$response = HTTPResponse::create(json_encode(['message' => $e->getMessage()]));
			$response->addHeader('Content-Type', 'application/json');
			throw new HTTPResponse_Exception($response, $e->getCode() ?: 500);
		}
	}

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function accessToken(): HTTPResponse
	{
		try {
			$settings = $this->getSettings();
			$provider = $settings->getProvider();
			$accessToken = $settings->getToken();

			if ($accessToken->hasExpired()) {
				$longLivedToken = $provider->getLongLivedAccessToken($accessToken);
				$refreshToken = $provider->getRefreshedAccessToken($longLivedToken);
				$settings->RefreshToken = $refreshToken->getToken();
				$settings->Expires = $refreshToken->getExpires();
				$settings->write();
			}
			return Api::constructResponse($this, [
				'accessCode' => $settings->RefreshToken,
				'userId' => $settings->ResourceOwnerId,
			]);
		} catch (Exception $e) {
			$response = HTTPResponse::create(json_encode(['message' => $e->getMessage()]));
			$response->addHeader('Content-Type', 'application/json');
			throw new HTTPResponse_Exception($response, HttpUtil::isValidStatusCode($e->getCode()) ?: 500);
		}
	}

	/**
	 * @return InstagramToken
	 * @throws Exception
	 */
	private function getSettings(): InstagramToken
	{
		$username = $this->getRequest()->param('Username');
		/** @var InstagramToken $token */
		$token = InstagramToken::get()->find('Username', $username);
		if ($token) {
			return $token;
		}
		throw new Exception("No Instagram settings exist for $username", 404);
	}

}
