<?php

namespace Fastrack\Core\Extension;

use Fastrack\Core\Model\EventRoom;
use Fastrack\Core\Model\Imgix;
use Fastrack\Core\Model\Offer;
use Fastrack\Core\Model\Property;
use Fastrack\Core\Model\Restaurant;
use Fastrack\Core\Model\Room;
use Fastrack\Core\Util\Slug;
use SilverStripe\AssetAdmin\Forms\UploadField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\FormField;
use SilverStripe\Forms\HeaderField;
use SilverStripe\Forms\HTMLEditor\HTMLEditorField;
use SilverStripe\Forms\LiteralField;
use SilverStripe\Forms\Tab;
use SilverStripe\Forms\TabSet;
use SilverStripe\Forms\TextareaField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataExtension;
use SilverStripe\ORM\DataObject;

/**
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @method Imgix Image()
 */
class ContentExtension extends DataExtension
{

	private static array $db = [
		'Slug' => 'Varchar',
		'Excerpt' => 'Text',
		'Content' => 'HTMLText'
	];

	private static array $has_one = [
		'Image' => Imgix::class,
	];

	private static array $owns = [
		'Image'
	];

	private static array $translate = [
		'Excerpt',
		'Content',
	];


	public function updateCMSFields(FieldList $fields): void
	{
		$folderName = match(get_class($this->getOwner())) {
			Property::class => $this->getOwner()->AssetFolder()?->getFilename(),
			EventRoom::class,
			Restaurant::class,
			Room::class => $this->getOwner()->Property()?->AssetFolder()?->getFilename(),
			Offer::class => $this->getOwner()->Properties()->count() === 1 ?
				$this->getOwner()->Properties()->first()?->AssetFolder()?->getFilename() : null,
			default => null,
		};

		$dataFields = [
			TextField::create('Slug'),
			UploadField::create('Image')
				->setFolderName($folderName ?? '')
				->setDisabled(!$this->getOwner()->exists()),
			TextareaField::create('Excerpt'),
			HTMLEditorField::create('Content'),
		];
		SecondaryTabSet::addFields($fields, $dataFields, 'Content');
	}

	public function onBeforeWrite(): void
	{
		$this->getOwner()->Slug = Slug::generateSlug($this->getOwner()->Title, $this->getOwner()->Slug);
		parent::onBeforeWrite();
	}

}
