<?php

namespace Fastrack\Core\Extension;

use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\CMS\Model\SiteTree;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\HeaderField;
use SilverStripe\Forms\NumericField;
use SilverStripe\Forms\TextField;
use SilverStripe\Forms\TreeDropdownField;
use SilverStripe\ORM\DataExtension;

class ExternalServicesExtension extends DataExtension
{

	private static array $db = [
		'GoogleMapAPIKey' => 'Varchar',
		'GoogleReCaptchaKey' => 'Varchar',
		'GoogleReCaptchaSecretKey' => 'Varchar',
		'BookingInterfaceUrl' => 'Varchar',
		'BookingInterfaceClient' => 'Int',
		'BookingInterfaceAgent' => 'Int',
	];

	private static array $has_one = [
		'BookingInterfaceRoomsPage' => SiteTree::class,
		'BookingInterfaceGuestDetailsPage' => SiteTree::class,
		'BookingInterfaceTermsAndConditionsPage' => SiteTree::class,
	];

	public function updateCMSFields(FieldList $fields): void
	{
		$fields->addFieldsToTab('Root.ExternalServices', [
			HeaderField::create('GoogleMaps', 'Google Maps'),
			TextField::create('GoogleMapAPIKey', 'Google Maps API Key'),
			HeaderField::create('GoogleReCaptcha', 'Google ReCaptcha'),
			TextField::create('GoogleReCaptchaKey', 'Google ReCaptcha Key'),
			TextField::create('GoogleReCaptchaSecretKey', 'Google ReCaptcha Secret Key'),
		]);

		$fields->addFieldsToTab('Root.ExternalServices', [
			HeaderField::create('BookingInterface', 'Booking Interface'),
			TextField::create('BookingInterfaceUrl', 'URL'),
			NumericField::create('BookingInterfaceClient', 'Client ID'),
			NumericField::create('BookingInterfaceAgent', 'Agent ID'),
			TreeDropdownField::create('BookingInterfaceRoomsPageID', 'Rooms page', SiteTree::class),
			TreeDropdownField::create('BookingInterfaceGuestDetailsPageID', 'Guest details page', SiteTree::class),
			TreeDropdownField::create('BookingInterfaceTermsAndConditionsPageID', 'Terms & conditions page', SiteTree::class),
		]);

		ModelUtilities::hideFields($this->getOwner(), $fields);
	}

}
