<?php

namespace Fastrack\Core\Extension;


use Fastrack\Core\Field\GoogleMapField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\TextField;
use SilverStripe\Forms\HiddenField;
use SilverStripe\Forms\TextareaField;
use SilverStripe\ORM\DataExtension;
use SilverStripe\ORM\DataObject;
use SilverStripe\SiteConfig\SiteConfig;

/**
 * @see LocationExtension
 * @property string Address1
 * @property string Address2
 * @property string City
 * @property string Postcode
 * @property string State
 * @property string Country
 * @property string Directions
 * @property string Latitude
 * @property string Longitude
 * @property string MapUrl
 */
class LocationExtension extends DataExtension
{
	private static array $db = [
		'Address1' => 'Varchar',
		'Address2' => 'Varchar',
		'City' => 'Varchar',
		'Postcode' => 'Varchar',
		'State' => 'Varchar',
		'Country' => 'Varchar',
		'Directions' => 'Text',
		'Latitude' => 'Varchar',
		'Longitude' => 'Varchar',
		'MapUrl' => 'Varchar',
	];

	private static array $translate = [
		'Address1',
		'Address2',
		'City',
		'Postcode',
		'State',
		'Country',
		'Directions',
	];


	public function updateCMSFields(FieldList $fields): void
	{
		$dataFields = [
			TextField::create('Address1', 'Address one'),
			TextField::create('Address2', 'Address two'),
			TextField::create('City'),
			TextField::create('Postcode'),
			TextField::create('State'),
			TextField::create('Country'),
			TextareaField::create('Directions'),
			TextField::create('MapUrl', 'Map URL'),
			HiddenField::create('Latitude'),
			HiddenField::create('Longitude'),
			GoogleMapField::create('Map'),
		];
		if ($this->getOwner() instanceof SiteConfig) {
			$fields->addFieldsToTab('Root.Location', $dataFields);
		} else {
			SecondaryTabSet::addFields($fields, $dataFields, 'Location');
		}
	}

	public function getAddressFormatted($country = false, $lineSeparator = PHP_EOL): string
	{
		$fields = [
			trim($this->getOwner()->Address1 ?: ''),
			trim($this->getOwner()->Address2 ?: ''),
			trim(join(' ', [$this->getOwner()->City, $this->getOwner()->State, $this->getOwner()->Postcode])),
			trim($country ? $this->getOwner()->Country ?? '' : ''),
		];
		$fields = array_filter($fields, function ($field) {return (bool)$field; });
		return join($lineSeparator, $fields);
	}

	public function getGoogleMapsLink(): string
	{
		if ($this->getOwner()->MapUrl) return $this->getOwner()->MapUrl;
		$url = 'https://www.google.com/maps/search/?api=1&query=';
		if ($this->getOwner()->Latitude && $this->getOwner()->Longitude) {
			$url .= $this->getOwner()->Latitude . ',' . $this->getOwner()->Longitude;
		} else {
			$url .= urlencode($this->getAddressFormatted(true, ''));
		}
		return $url;
	}

}
