<?php

namespace Fastrack\Core\Extension;

use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\HiddenField;
use SilverStripe\ORM\DataExtension;
use SilverStripe\ORM\DataQuery;
use SilverStripe\ORM\Queries\SQLSelect;
use SilverStripe\Subsites\Model\Subsite;
use SilverStripe\Subsites\State\SubsiteState;

/**
 * Almost a direct copy from Copied from SilverStripe\Subsites\Extensions\SiteConfigSubsites
 *
 * @see SiteConfigSubsites
 */
class RedirectURLsExtension extends DataExtension
{

	private static array $has_one = [
		'Subsite' => Subsite::class,
	];

	public function augmentSQL(SQLSelect $query, DataQuery $dataQuery = null): void
	{
		if (Subsite::$disable_subsite_filter) {
			return;
		}

		if ($query->filtersOnID()) {
			return;
		}

		$regexp = '/^(.*\.)?("|`)?SubsiteID("|`)?\s?=/';
		foreach ($query->getWhereParameterised($parameters) as $predicate) {
			if (preg_match($regexp, $predicate ?? '')) {
				return;
			}
		}

		$subsiteID = SubsiteState::singleton()->getSubsiteId();
		if ($subsiteID === null) {
			return;
		}

		$from = $query->getFrom();
		$from = array_keys($from ?? []);
		$tableName = array_shift($from);
		if ($tableName !== $this->owner->baseTable()) {
			return;
		}
		$query->addWhere("\"$tableName\".\"SubsiteID\" IN ($subsiteID)");
	}

	public function onBeforeWrite(): void
	{
		if ((!is_numeric($this->owner->ID) || !$this->owner->ID) && !$this->owner->SubsiteID) {
			$this->owner->SubsiteID = SubsiteState::singleton()->getSubsiteId();
		}
	}

	public function updateCMSFields(FieldList $fields): void
	{
		$fields->push(HiddenField::create('SubsiteID', 'SubsiteID', SubsiteState::singleton()->getSubsiteId()));
	}

}
