<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string Title
 * @property string Key
 * @method ManyManyList CalendarEvents()
 */
class CalendarEventCategory extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_CALENDAR_EVENT_CATEGORIES';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static string $table_name = 'CalendarEventCategory';

	private static array $db = [
		'Title' => 'Varchar',
		'Key' => 'Varchar'
	];

	private static array $belongs_many_many = [
		'CalendarEvents' => CalendarEvent::class,
	];

	private static array $translate = [
		'Title',
	];

	private static array $hidden_fields = [];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage calendar event categories',
				'category' => 'Calendar events',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
