<?php

namespace Fastrack\Core\Model;

use DOMDocument;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\LiteralField;
use SilverStripe\Forms\ReadonlyField;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;

/**
 * @property string $To
 * @property string $From
 * @property string $Body
 */
class Mail extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_MAIL';

	private static string $table_name = 'Mail';

	private static array $db = [
		'To' => 'Varchar',
		'From' => 'Varchar',
		'Body' => 'Text',
	];

	private static string $plural_name = 'Mail';

	private static array $field_labels = [
		'Created' => 'Sent'
	];

	private static array $summary_fields = [
		'From',
		'To',
		'Created',
	];

	private static array $hidden_fields = [];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$fields->addFieldToTab('Root.Main', LiteralField::create('Body', $this->getBodyContentsOfEmail()));
		$fields->addFieldToTab('Root.Main', ReadonlyField::create('Created', 'Sent', $this->Created), 'To');
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage mail',
				'category' => 'Mail',
				'sort' => -100,
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool
	{
		return false;
	}

	private function getBodyContentsOfEmail(): string
	{
		$document = new DOMDocument();
		$contents = new DOMDocument();
		$document->loadHTML($this->Body);
		$body = $document->getElementsByTagName('body')->item(0);
		foreach ($body->childNodes as $childNode) {
			$contents->appendChild($contents->importNode($childNode, true));
		}
		return '<div class="form-group"><label class="form__field-label">Message</label>'
			. $contents->saveHTML() . '</div>';
	}

}
