<?php


namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $Content
 * @method Imgix Image()
 * @method HasManyList Places()
 */
class PlaceCategory extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_PLACE_CATEGORY';

	private static string $table_name = 'PlaceCategory';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Content' => 'HTMLText',
	];

	private static array $has_one = [
		'Image' => Imgix::class,
	];

	private static array $belongs_many_many = [
		'Places' => Place::class,
	];

	private static array $translate = [
		'Title',
	];

	private static array $owns = [
		'Image'
	];

	private static array $hidden_fields = [];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$placesConfig = GridFieldConfig_RelationEditor::create();
		$placesConfig->removeComponentsByType(GridFieldAddNewButton::class);
		$placesConfig->addComponent(new GridFieldAddExistingSearchButton());
		$placesGrid = GridField::create('Places', 'Places', $this->Places(), $placesConfig);
		$fields->replaceField('Places', $placesGrid);
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage place categories',
				'category' => 'Places',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
