<?php


namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\AssetAdmin\Forms\UploadField;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $Review
 * @property string $Date
 * @property string $Author
 * @property int $Sort
 * @method Property Property()
 * @method Imgix Image()
 */
class Review extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_REVIEWS';

	private static string $table_name = 'Review';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Content' => 'HTMLText',
		'Date' => 'Date',
		'Author' => 'Varchar',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
		'Image' => Imgix::class,
	];

	private static array $owns = [
		'Image',
	];

	private static array $indexes = [
		'Sort' => true,
	];

	private static array $translate = [
		'Title',
		'Content',
		'Date',
		'Author',
	];

	private static string $default_sort = 'Sort';

	private static array $hidden_fields = [
		'Sort',
		'PropertyID',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();

		/** @var UploadField $image */
		$image = $fields->dataFieldByName('Image');
		if (!$this->exists()) {
			$image->setDisabled(true);
			$image->setDescription('You must create the review before adding an image');
		}
		if ($this->Property()->exists()) {
			$image->setFolderName($this->Property()->AssetFolder()?->getFilename() ?? '');
		}

		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage reviews',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
