<?php

namespace Fastrack\Core\Util;

use League\Flysystem\FileAttributes;

class CachedFile
{

	private int $exists;
	private FileAttributes $fileAttributes;
	private string $contents;
	private string $url;

	public function __construct(int $exists, FileAttributes $fileAttributes, string $contents = '', $url = '')
	{
		$this->exists = $exists;
		$this->contents = $contents;
		$this->fileAttributes = $fileAttributes;
		$this->url = $url;
	}

	public function exists(): int
	{
		return $this->exists;
	}

	public function getContents(): string
	{
		return $this->contents;
	}

	public function getFileAttributes(): FileAttributes
	{
		return $this->fileAttributes;
	}

	public function url()
	{
		return $this->url;
	}

	public function serialize(): string
	{
		return json_encode([
			'exists' => $this->exists,
			'path' => $this->fileAttributes->path(),
			'fileSize' => $this->fileAttributes->fileSize(),
			'visibility' => $this->fileAttributes->visibility(),
			'lastModified' => $this->fileAttributes->lastModified(),
			'mimeType' => $this->fileAttributes->mimeType(),
			'extraMetadata' => $this->fileAttributes->extraMetadata(),
		]);
	}

	public static function deserialize(string $serialized): self
	{
		$data = json_decode($serialized, true);
		$fileAttributes = new FileAttributes(
			$data['path'],
			$data['fileSize'],
			$data['visibility'],
			$data['lastModified'],
			$data['mimeType'],
			$data['extraMetadata'],
		);
		return new CachedFile($data['exists'], $fileAttributes);
	}


}
