<?php


namespace Fastrack\Core\Util;

use SilverStripe\Core\Config\Config;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddExistingAutocompleter;
use Symbiote\AdvancedWorkflow\Admin\AdvancedWorkflowAdmin;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;

class ModelUtilities
{

	private static bool $show_workflow_fields = false;

	const ALL_WORKFLOW_FIELDS = 'ALL_WORKFLOW_FIELDS';

	const PRESETS = [
		self::ALL_WORKFLOW_FIELDS => [
			'Workflow',
			'AdditionalWorkflowDefinitions',
		],
	];

	public static function addOrderRows(FieldList $fields, array $fieldNames): void
	{
		$gridFields = self::getGridFields($fields, $fieldNames);
		array_walk($gridFields, fn(GridField $f) => $f->getConfig()->addComponent(GridFieldOrderableRows::create()));
	}

	public static function removeLinkExisting(FieldList $fields, array $fieldNames): void
	{
		$gridFields = self::getGridFields($fields, $fieldNames);
		array_walk($gridFields,
			fn(GridField $f) => $f->getConfig()->removeComponentsByType(GridFieldAddExistingAutocompleter::class));
	}

	/**
	 * @param FieldList $fields
	 * @param array $fieldNames
	 * @return GridField[]
	 */
	private static function getGridFields(FieldList $fields, array $fieldNames): array
	{
		$gridFields = [];
		foreach ($fieldNames as $fieldName) {
			$field = $fields->dataFieldByName($fieldName);
			if ($field instanceof GridField) {
				$gridFields[] = $field;
			}
		}
		return $gridFields;
	}

	public static function hideFields($class, FieldList $fields): void
	{
		$fieldNames = [];

		$hiddenFields = Config::inst()->get($class, 'hidden_fields');
		if ($hiddenFields && is_array($hiddenFields)) {
			$fieldNames = array_merge($fieldNames, $hiddenFields);
		}
		foreach ($fieldNames as $fieldKey) {
			if (array_key_exists($fieldKey, self::PRESETS)) {
				$preset = self::PRESETS[$fieldKey];
				array_walk($preset, fn($f) => self::hideField($fields, $f));
			} else {
				self::hideField($fields, $fieldKey);
			}
		}
	}

	private static function hideField(FieldList $fieldList, string $fieldKey): void
	{
		if (str_contains($fieldKey, '.')) {
			$components = explode('.', $fieldKey);
			$field = [array_pop($components)];
			$fieldList->removeFieldsFromTab(implode('.', $components), $field);
		} else {
			$fieldList->removeByName($fieldKey);
		}
	}

}
