<?php

namespace Fastrack\Core\Util\PaymentGateway;

use Exception;

class Transactions
{

	/** @var array<Transaction>  */
	private array $transactions;

	/**
	 * @param array $transactions
	 */
	public function __construct(array $transactions)
	{
		$this->transactions = [];
		foreach ($transactions as $transaction) {
			$this->transactions[] = new Transaction($transaction);
		}
	}

	/**
	 * @throws Exception
	 */
	public function getTransaction(): Transaction
	{
		foreach ($this->transactions as $transaction) {
			if (!$transaction->getIsSurcharge()) {
				return $transaction;
			}
		}
		if (count($this->transactions)) {
			return $this->transactions[0];
		}
		throw new Exception('No payment gateway transactions available');
	}

	/**
	 * @throws Exception
	 */
	public function getTotal(): int
	{
		$transaction = $this->getTransaction();
		$amount = floatval($transaction->getAmount());
		$surcharge = floatval($transaction->getAmountSurcharge());
		return intval(($amount + $surcharge) * 100);
	}

}
