<?php

namespace Fastrack\Core\Util\PaymentGateway;

use Exception;
use Fastrack\Core\Model\PaymentGateway;
use Fastrack\Core\Util\HttpUtil;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\GuzzleException;

class Windcave
{

	/**
	 * @throws Exception
	 */
	public function createSession(PaymentGateway $paymentGateway, CreateSessionRequest $request): array
	{
		try {
			$response = HttpUtil::buildClient(true)->request('POST', $paymentGateway->getUrl(), [
				'body' => json_encode([
					'type' => $request->getType(),
					'amount' => $request->getAmount(),
					'currency' => $request->getCurrency(),
					'merchantReference' => $request->getMerchantReference(),
					'language' => 'en',
					'methods' => ['card'],
					'amountSurcharge' => $request->getSurcharge(),
					'storeCard' => true,
					'callbackUrls' => [
						'approved' => $request->getApproved(),
						'declined' => $request->getDeclined(),
						'cancelled' => $request->getCancelled(),
					],
					'customer' => [
						'firstName' => $request->getFirstName(),
						'lastName' => $request->getLastName(),
						'email' => $request->getEmail(),
						'phoneNumber' => $request->getPhoneNumber(),
					],
				]),
				'headers' => [
					'Content-Type' => 'application/json',
				],
				'auth' => [$paymentGateway->Username, $paymentGateway->AccessKey],
			]);
			$data = json_decode($response->getBody()->getContents());
			foreach ($data->links as $link) {
				if ($link->rel === 'hpp') {
					return [
						'hpp' => $link->href,
						'session' => $data->id,
					];
				}
			}
			throw new Exception('Could not redirect to payment gateway');
		} catch (GuzzleException|ClientException) {
			throw new Exception('Could not redirect to payment gateway');
		}
	}

	/**
	 * @throws Exception
	 */
	public function transactionResult(PaymentGateway $paymentGateway, string $sessionId): Transactions
	{
		try {
			$url = join('/', [$paymentGateway->getUrl(), $sessionId]);
			$response = HttpUtil::buildClient(true)->request('GET', $url, [
				'auth' => [$paymentGateway->Username, $paymentGateway->AccessKey],
			])->getBody()->getContents();
			$data = json_decode($response);
			if (count($data->transactions)) {
				return new Transactions($data->transactions);
			}
			throw new Exception("Could not find transaction");
		} catch (GuzzleException|ClientException) {
			throw new Exception("Could not obtain payment gateway's transaction result");
		}
	}

}
