<?php

namespace Fastrack\Core\Extension;

use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\FormField;
use SilverStripe\Forms\Tab;
use SilverStripe\Forms\TabSet;
use SilverStripe\ORM\DataExtension;

class SecondaryTabSet extends DataExtension
{

	const TAB_NAME = 'Secondary';
	const GENERAL_TAB = 'General';

	public static function addFields(FieldList $fields, array $newFields, string $tabName = self::GENERAL_TAB): void
	{
		$fields->removeByName(array_map(fn(FormField $field) => $field->getName(), $newFields));
		$tab = self::getTab($fields, $tabName);
		array_walk($newFields, fn(FormField $field) => $tab->push($field));
	}

	public static function getTab(FieldList $fields, string $name): Tab
	{
		$secondaryTab = self::getSecondaryTab($fields);
		$tab = $secondaryTab->Tabs()->fieldByName($name);
		if (!$tab) {
			$tab = Tab::create($name);
			$secondaryTab->push($tab);
		}
		return $tab;
	}

	private static function getSecondaryTab(FieldList $fields): TabSet
	{
		$tabSet = $fields->fieldByName('Root.Main.' . self::TAB_NAME);
		if (!$tabSet) {
			$tabSet = TabSet::create(self::TAB_NAME);
			$fields->addFieldToTab('Root.Main', $tabSet);
		}
		return $tabSet;
	}

}
